#include "bits/stdc++.h"

using namespace std;

using ll = long long;

struct upd_node {
    int8_t b_set;
    bool invert;

    upd_node() : b_set(-1), invert(false) {}

    void merge(const upd_node &parent) {
        if (parent.b_set != -1) {
            invert = false;
            b_set = parent.b_set;
        } else if (parent.invert) {
            if (b_set != -1) {
                b_set = static_cast<int8_t>(1 - b_set);
            } else {
                invert ^= parent.invert;
            }
        }
    }
};


struct bit_stree {
    vector<int> sum;
    vector<upd_node> upd;
    int n;

    bit_stree(int n, vector<int>:: const_iterator &it) : n(n), sum(2 * n), upd(2 * n) {
        build(1, 0, n - 1, it);
    }

    bit_stree(): sum(), upd(), n(0) {}

    void build(int v, int l, int r, vector<int>::const_iterator& it) {
        if (l == r) {
            sum[v] = *it;
            it++;
            return;
        }
        int mid = (l + r) / 2;
        int l_child = v << 1;
        build(l_child, l, mid, it);
        build(l_child + 1, mid + 1, r, it);
        sum[v] = sum[l_child] + sum[l_child + 1];
    }

    void push(int v, int len) {
        if (len > 1) {
            upd[v << 1].merge(upd[v]);
            upd[(v << 1) + 1].merge(upd[v]);
        }
        if (upd[v].b_set != -1) {
            sum[v] = len * upd[v].b_set;
            upd[v].b_set = -1;
        } else if (upd[v].invert) {
            sum[v] = len - sum[v];
            upd[v].invert = false;
        }
    }

    void set(int v, int l, int r, int ql, int qr, int8_t val) {
        push(v, r - l + 1);
        if (ql <= l and r <= qr) {
            upd[v].b_set = val;
            push(v, r - l + 1);
            return;
        }
        if (l > qr or r < ql) {
            return;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        set(l_child, l, mid, ql, qr, val);
        set(l_child + 1, mid + 1, r, ql, qr, val);
        sum[v] = sum[l_child + 1] + sum[l_child];
    }

    void inverse(int v, int l, int r, int ql, int qr) {
        push(v, r - l + 1);
        if (ql <= l and r <= qr) {
            upd[v].invert = true;
            push(v, r - l + 1);
            return;
        }
        if (l > qr or r < ql) {
            return;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        inverse(l_child, l, mid, ql, qr);
        inverse(l_child + 1, mid + 1, r, ql, qr);
        sum[v] = sum[l_child + 1] + sum[l_child];
    }

    int _sum(int v, int l, int r, int ql, int qr) {
        push(v, r - l + 1);
        if (ql <= l and r <= qr) {
            return sum[v];
        }
        if (r < ql or l > qr) {
            return 0;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        return _sum(l_child, l, mid, ql, qr) + _sum(l_child + 1, mid + 1, r, ql, qr);
    }

    int sum_q(int l, int r) {
        return _sum(1, 0, n - 1, l, r);
    }

    void set(int l, int r, int8_t val) {
        set(1, 0, n - 1, l, r, val);
    }

    void inverse(int l, int r) {
        inverse(1, 0, n - 1, l, r);
    }

    int get_sum() {
        push(1, n);
        return sum[1];
    }
};

struct xor_stree {
    int n;
    int cnt_leaf = 0;
    vector<bit_stree> leafs;
    vector<int> sum, leaf_index; // leaf_type \in {-1(not leaf), 0(no inverse), 1(inverse)}
    vector<upd_node> upd;

    xor_stree(vector<int>& arr, int bit_level) { // arr should be 1-indexed
        n = arr.size();
        leafs.resize(max(1, n >> bit_level));
        sum.resize(2 * n);
        leaf_index.resize(2 * n, -1);
        upd.resize(2 * n);
        build(1, 0, n - 1, (int) min((ll) n, 1LL << bit_level), arr);
    }

    void build(int v, int l, int r, int stop_len, const vector<int>& arr) {
        if (r - l + 1 <= stop_len) {
            auto it = arr.begin() + l;
            leafs[cnt_leaf] = bit_stree(stop_len, it);
            leaf_index[v] = cnt_leaf;
            sum[v] = eval_leaf_sum(v, stop_len, leafs[cnt_leaf].get_sum());
            cnt_leaf++;
            return;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        build(l_child, l, mid, stop_len, arr);
        build(l_child + 1, mid + 1, r, stop_len, arr);
        sum[v] = sum[l_child + 1] + sum[l_child];
    }

    int eval_leaf_sum(int v, int len, int s) {
        return ((leaf_index[v] & 1) == 0) ? s : len - s;
    }

    void push(int v, int len) {
        if (leaf_index[v] == -1) {
            upd[v << 1].merge(upd[v]);
            upd[(v << 1) + 1].merge(upd[v]);
        }
        int l_ind = leaf_index[v];
        if (upd[v].b_set != -1) {
            sum[v] = len >> 1;
            if (l_ind != -1) {
                leafs[l_ind].set(0, len - 1, upd[v].b_set);
                sum[v] = eval_leaf_sum(v, len, leafs[l_ind].get_sum());
            }
            upd[v].b_set = -1;
        } else if (upd[v].invert) {
            sum[v] = len - sum[v];
            upd[v].invert = false;
            if (l_ind != -1) {
                leafs[l_ind].inverse(0, len - 1);
                sum[v] = eval_leaf_sum(v, len, leafs[l_ind].get_sum());
            }
        }
    }

    int q_sum(int v, int l, int r, int ql, int qr) {
        push(v, r - l + 1);
        if (l > qr or r < ql) {
            return 0;
        }
        if (ql <= l and r <= qr) {
            return sum[v];
        }
        int l_i = leaf_index[v];
        if (l_i != -1) {
            int s = leafs[l_i].sum_q(max(l, ql) - l, min(r, qr) - l);
            return eval_leaf_sum(v, min(r, qr) - max(l, ql) + 1, s);
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        return q_sum(l_child, l, mid, ql, qr) + q_sum(l_child + 1, mid + 1, r, ql, qr);
    }

    void q_set(int v, int l, int r, int ql, int qr, int val) {
        push(v, r - l + 1);
        if (l > qr or r < ql) {
            return;
        }
        if (ql <= l and r <= qr) {
            upd[v].b_set = val;
            push(v, r - l + 1);
            return;
        }
        int l_i = leaf_index[v];
        if (l_i != -1) {
            leafs[l_i].set(max(l, ql) - l, min(r, qr) - l, val);
            sum[v] = eval_leaf_sum(v, r - l + 1, leafs[l_i].get_sum());
            return;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        q_set(l_child, l, mid, ql, qr, val);
        q_set(l_child + 1, mid + 1, r, ql, qr, val);
        sum[v] = sum[l_child] + sum[l_child + 1];
    }

    void q_inverse(int v, int l, int r, int ql, int qr) {
        push(v, r - l + 1);
        if (l > qr or r < ql) {
            return;
        }
        if (ql <= l and r <= qr) {
            upd[v].invert = true;
            push(v, r - l + 1);
            return;
        }
        int li = leaf_index[v];
        if (li != -1) {
            leafs[li].inverse(max(l, ql) - l, min(r, qr) - l);
            sum[v] = eval_leaf_sum(v, r - l + 1, leafs[li].get_sum());
            return;
        }
        int mid = (l + r) >> 1;
        int l_child = v << 1;
        q_inverse(l_child, l, mid, ql, qr);
        q_inverse(l_child + 1, mid + 1, r, ql, qr);
        sum[v] = sum[l_child] + sum[l_child + 1];
    }

    ll q_sum(int l, int r) {
        return q_sum(1, 0, n - 1, l, r);
    }

    void update(int l, int r, char c, int x) {
        if (c == '=') {
            q_set(1, 0, n - 1, l, r, x);
        } else if (c == '^' and x == 1) {
            q_inverse(1, 0, n - 1, l, r);
        } else if ((c == '|' and x == 1) or (c == '&' and x == 0)) {
            q_set(1, 0, n - 1, l, r, x);
        }
    }
};

void solve() {
    const int BITS = 17;
    int n, m;
    cin >> n >> m;
    vector<int> arr(n + 1);
    for (int i = 1; i <= n; i++) {
        cin >> arr[i];
    }
    int new_n = 1;
    while (new_n <= (n + 1)) {
        new_n <<= 1;
    }
    arr.resize(new_n);
    vector<xor_stree> trees;
    vector<int> bits(new_n);
    for (int bit = 0; bit < BITS; bit++) {
        for (int i = 1; i <= n; i++) {
            bits[i] = ((arr[i] >> bit) & 1);
        }
        trees.emplace_back(bits, bit);
    }
    while (m--) {
        int t, l, r;
        cin >> t >> l >> r;
        if (t == 1) {
            ll answer = 0;
            for (ll bit = 0; bit < BITS; bit++) {
                answer += (ll)(trees[bit].q_sum(l, r)) << bit;
            }
            cout << answer << "\n";
        } else {
            char cmd = '=';
            int x;
            cin >> x;
            if (t == 3) {
                cin >> cmd;
            }
            for (int bit = 0; bit < BITS; bit++) {
                trees[bit].update(l, r, cmd, (x >> bit) & 1);
            }
        }
    }
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    solve();
    return 0;
}